"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createServer = void 0;
const cron_handler_1 = require("./cron-handler");
const body_parser_1 = __importDefault(require("body-parser"));
const express_1 = __importDefault(require("express"));
const http_1 = __importDefault(require("http"));
const socket_io_1 = require("socket.io");
const flows_endpoint_1 = require("./flows-endpoint");
const guards_1 = require("./guards");
const logger_1 = require("./logger");
const steps_1 = require("./steps");
const call_step_file_1 = require("./call-step-file");
const LoggerFactory_1 = require("./LoggerFactory");
const generate_trace_id_1 = require("./generate-trace-id");
const flows_config_endpoint_1 = require("./flows-config-endpoint");
const middleware_composer_1 = __importDefault(require("./middleware-composer"));
const createServer = async (lockedData, eventManager, state, config) => {
    const printer = lockedData.printer;
    const app = (0, express_1.default)();
    const server = http_1.default.createServer(app);
    const io = new socket_io_1.Server(server);
    const loggerFactory = new LoggerFactory_1.LoggerFactory(config.isVerbose, io);
    const allSteps = [...steps_1.systemSteps, ...lockedData.activeSteps];
    const cronManager = (0, cron_handler_1.setupCronHandlers)(lockedData, eventManager, state, loggerFactory);
    const asyncHandler = (step) => {
        return async (req, res) => {
            const traceId = (0, generate_trace_id_1.generateTraceId)();
            const { name: stepName, flows } = step.config;
            const logger = loggerFactory.create({ traceId, flows, stepName });
            logger.debug('[API] Received request, processing step', { path: req.path });
            const request = {
                body: req.body,
                headers: req.headers,
                pathParams: req.params,
                queryParams: req.query,
            };
            const ctx = {
                emit: async (event) => {
                    await eventManager.emit({
                        topic: event.topic,
                        data: event.data,
                        traceId,
                        logger,
                    });
                },
                traceId,
                state,
                logger,
            };
            const finalHandler = async () => {
                try {
                    const result = await (0, call_step_file_1.callStepFile)({
                        contextInFirstArg: false,
                        data: request,
                        step,
                        printer,
                        logger,
                        eventManager,
                        state,
                        traceId,
                    });
                    if (!result) {
                        return { status: 500, body: { error: 'Internal server error' } };
                    }
                    return result;
                }
                catch (error) {
                    logger.error('[API] Internal server error', { error });
                    console.log(error);
                    return { status: 500, body: { error: 'Internal server error' } };
                }
            };
            try {
                const middleware = step.config.middleware || [];
                const result = await (0, middleware_composer_1.default)(...middleware)(request, ctx, finalHandler);
                if (result.headers) {
                    Object.entries(result.headers).forEach(([key, value]) => res.setHeader(key, value));
                }
                res.status(result.status);
                res.json(result.body);
            }
            catch (error) {
                logger.error('[API] Error in middleware chain', { error });
                res.status(500).json({ error: 'Internal server error' });
            }
        };
    };
    app.use(body_parser_1.default.json());
    app.use(body_parser_1.default.urlencoded({ extended: true }));
    const router = express_1.default.Router();
    const addRoute = (step) => {
        const { method, path } = step.config;
        logger_1.globalLogger.debug('[API] Registering route', step.config);
        const expressHandler = asyncHandler(step);
        const methods = {
            GET: () => router.get(path, expressHandler),
            POST: () => router.post(path, expressHandler),
            PUT: () => router.put(path, expressHandler),
            DELETE: () => router.delete(path, expressHandler),
            PATCH: () => router.patch(path, expressHandler),
            OPTIONS: () => router.options(path, expressHandler),
            HEAD: () => router.head(path, expressHandler),
        };
        const methodHandler = methods[method];
        if (!methodHandler) {
            throw new Error(`Unsupported method: ${method}`);
        }
        methodHandler();
    };
    const removeRoute = (step) => {
        const { path, method } = step.config;
        const routerStack = router.stack;
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        const filteredStack = routerStack.filter((layer) => {
            if (layer.route) {
                const match = layer.route.path === path && layer.route.methods[method.toLowerCase()];
                return !match;
            }
            return true;
        });
        router.stack = filteredStack;
    };
    allSteps.filter(guards_1.isApiStep).forEach(addRoute);
    app.use(router);
    (0, flows_endpoint_1.flowsEndpoint)(lockedData, app);
    (0, flows_config_endpoint_1.flowsConfigEndpoint)(app, process.cwd());
    server.on('error', (error) => {
        console.error('Server error:', error);
    });
    const close = async () => {
        cronManager.close();
        await io.close();
        server.close();
    };
    return { app, server, socketServer: io, close, removeRoute, addRoute, cronManager };
};
exports.createServer = createServer;
